@@ -13,8 +13,8 @@
 # You should have received a copy of the GNU General Public License
 # along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 #
-# Copyrigth 2014, Ogier Maitre, Andre Berchtold 
-# ogier.maitre@epfl.ch andre.berchtold@unil.ch 
+# Copyrigth 2014-2020, Ogier Maitre, Kevin Emery, André Berchtold 
+# andre.berchtold@unil.ch 
 #
 
 
@@ -26,25 +26,24 @@
 
 # show method for model (abstract) object
 march.model.show <- function(object){
-  cat("ll : ",object@ll,"\n")
-  cat("dsL: ",object@dsL,"\n")
+  cat("Log-likelihood : ",object@ll,"\n")
+  cat("Number of data : ",object@dsL,"\n")
 }
 
 # show method for indep object
 march.indep.show <- function(object){
   cat(march.name(object))
-  cat("\n")
+  cat("\n\nProbability distribution :\n")
   
   s <- array(0,c(1,length(object@indP)))
   colnames(s) <- 1:object@y@K
   
   s[1,] <- object@indP
-  rownames(s) <- c("indP")
   print(s)
-  cat("\n")
+  
+  cat("\nFrequency distribution :\n")
   
   s[1,] <- object@indC
-  rownames(s) <- c("indC")
   print(s)
   cat("\n")
   
@@ -54,18 +53,17 @@ march.indep.show <- function(object){
 # show method for mc object
 march.mc.show <- function(object){
   cat(march.name(object))
-  cat("\nRC : \n")
+  cat("\n\nTransition matrix of order", object@order, ": \n")
   
   s <- march.h.mc.printableMatrix(object@RC,object@order,object@y@K)
   print(s)
   cat("\n")
   
-  cat("RT : \n")
+  cat("Crosstable : \n")
   s <- march.h.mc.printableMatrix(object@RT,object@order,object@y@K)
   print(s)
   cat("\n")
-  cat("order : ",object@order,"\n")
-  
+
   march.model.show(object)
 }
 
@@ -74,82 +72,175 @@ march.mtd.show <- function(object){
   placeCovar <- which(object@MCovar==1)
   
   cat(march.name(object))
-  cat("\n")
-  cat("Transition matrix : \n")
+  cat("\n\n")
+  cat("High-order transition matrix : \n")
   s <- march.cov.h.mc.printableMatrix(object@RA,object@order,object@y@K,object@y@Kcov[placeCovar],sum(object@MCovar))
   print(s)
   cat("\n")
-  # if( dim(object@Q)[1]==1 ){
-  #   
-  #   cat("Q : \n")
-  #   for( i in 1:object@y@K ){
-  #     cat(sprintf("%.4f", object@Q[1,i,]))
-  #     cat("\n")
-  #   }
-  # }
-  # else{
-  #   for( g in 1:dim(object@Q)[1]){
-  #     cat(sprintf("Q%d : \n",g))
-  #     for( i in 1:object@y@K ){
-  #       cat(sprintf("%.4f", object@Q[g,i,]))
-  #       cat("\n")
-  #     }    
-  #   }
-  # }
-  # cat("\n")
-  # if(sum(object@MCovar)>0){
-  #   for(g in 1:sum(object@MCovar)){
-  #     cat(sprintf("S%d : \n",g))
-  #     for(i in 1:object@y@Kcov[placeCovar[g]]){
-  #       cat(sprintf("%.4f", object@S[[g]][i,]))
-  #       cat("\n")
-  #     }
-  #   }
-  # }
-  # cat("\n")
-  # cat("phi : ")
-  # cat(sprintf("%.4f",object@phi))
-  # cat("\n\n")
-  # cat("order : ",object@order,"\n")
+  
+  # MTD model
+  cat("Vector of weights : \n")
+  print(object@phi)
+  cat("\n")
+  
+  for (i in 1:(dim(object@Q)[1])){
+    if (i==1){
+      if (dim(object@Q)[1]==1){
+        cat("Transition matrix : \n")
+      } else {
+        cat("Transition matrix, lag",i,": \n")        
+      }
+    } else {
+      cat("Transition matrix, lag",i,": \n")        
+    }
+    print(object@Q[i,,])
+    cat("\n")    
+  }
+  
+  if (length(object@phi)>object@order){
+    for (i in 1:sum(object@MCovar)){
+      cat("Transition matrix for covariate",i,": \n")        
+      print(object@S[i])
+    }
+  }
+  
   march.model.show(object)
 }
 
 march.dcmm.show <- function(object){
 	  placeACovar <- which(object@AMCovar==1)
 	  placeCCovar <- which(object@CMCovar==1)
+	  
 	  AtmCovar <- 1
   	if(sum(object@AMCovar>0)){
   		AtmCovar <-prod(object@y@Kcov[placeACovar])
   	}
-	
-  	cat(march.name(object))
-  	cat("\nRA : \n")
-  	if(object@orderHC==0){
-  		s <- march.cov.h.mc.printableMatrix(object@A[1:AtmCovar,,drop=FALSE],0,object@M,object@y@Kcov[placeACovar],sum(object@AMCovar))
+
+	  CtmCovar <- 1
+	  if(sum(object@CMCovar>0)){
+	    CtmCovar <-prod(object@y@Kcov[placeCCovar])
+	  }
+	  
+  	cat(march.name(object),"\n\n")
+  	
+  	if (object@M==1 & sum(object@AMCovar)==0){
+  	  cat("No hidden level \n\n")
   	}else{
-  		s <- march.cov.h.mc.printableMatrix(march.dcmm.cov.h.compactA(object),object@orderHC,object@M,object@y@Kcov[placeACovar],sum(object@AMCovar))
+  	  cat("Hidden level: \n")
+  	  cat("=============  \n\n")
+  	  
+  	  for( i in 1:object@orderHC){
+  	    cat("Distribution of hidden state",i,": \n")
+  	    for( j in 1:(AtmCovar*object@M^(i-1))){
+  	      cat(object@Pi[j,,i])
+  	      cat("\n")
+  	    }
+  	    cat("\n")
+  	  }
+  	  
+  	  cat("Hidden transition matrix : \n")
+  	  if(object@orderHC==0){
+  	    s <- march.cov.h.mc.printableMatrix(object@A[1:AtmCovar,,drop=FALSE],0,object@M,object@y@Kcov[placeACovar],sum(object@AMCovar))
+  	  }else{
+  	    s <- march.cov.h.mc.printableMatrix(march.dcmm.cov.h.compactA(object),object@orderHC,object@M,object@y@Kcov[placeACovar],sum(object@AMCovar))
+  	  }
+  	  print(s)
+  	  cat("\n") 	  
+  	  
+  	  
+  	  if(sum(object@AMCovar)>0 | (object@Amodel=="mtd" & object@orderHC>1) | (object@Amodel=="mtdg" & object@orderHC>1)){
+  	    # Display the modeling of the hidden transition matrix
+  	    # Rem: Covariates are allowed at the hidden level only when the hidden order is >0
+  	    #      and there are at least two hidden states
+  	    
+  	    cat("Modeling of the hidden transition matrix : \n\n")
+  	    
+  	    cat("Vector of weights : \n")
+  	    print(object@APhi)
+  	    cat("\n")
+  	    
+  	    for (i in 1:(dim(object@AQ)[1])){
+  	      if (i==1){
+  	        if (dim(object@AQ)[1]==1){
+  	          cat("Transition matrix : \n")
+  	        } else {
+  	          cat("Transition matrix, lag",i,": \n")        
+  	        }
+  	      } else {
+  	        cat("Transition matrix, lag",i,": \n")        
+  	      }
+  	      print(object@AQ[i,,])
+  	      cat("\n")    
+  	    }
+  	    
+  	    if (sum(object@AMCovar)>0){
+  	      for (i in 1:sum(object@AMCovar)){
+  	        cat("Transition matrix for covariate",i,": \n")        
+  	        print(object@ATCovar[i])
+  	        cat("\n")  
+  	      }
+  	    }
+  	    
+  	  }
+  	  
+  	  
   	}
-  	print(s)
-  	cat("\n")
-  
-  	cat("RB : \n")
+  	
+  	cat("Visible level : \n")
+  	cat("=============== \n\n")
+  	
    	for( i in 1:object@M ){
-     	cat(i,":\n");
+   	  if (object@M>1){
+   	    cat("Model corresponding to hidden state", i, ":\n")
+   	    cat("------------------------------------- \n\n")
+   	  }
+   	  
    	  if(object@orderVC==0){
-   	    s <- march.cov.h.mc.printableMatrix.orderVC0(matrix(object@RB[,,i],AtmCovar,object@y@K),object@y@K,object@y@Kcov[placeCCovar],sum(object@CMCovar))
+   	    s <- march.cov.h.mc.printableMatrix.orderVC0(matrix(object@RB[,,i],CtmCovar,object@y@K),object@y@K,object@y@Kcov[placeCCovar],sum(object@CMCovar))
    	  }else{
         s <- march.cov.h.mc.printableMatrix(object@RB[,,i],object@orderVC,object@y@K,object@y@Kcov[placeCCovar],sum(object@CMCovar))
    	  }
    	  print(s)
-    }  
-  	cat("\nPi : \n")
-  	for( i in 1:object@orderHC){
-    	for( j in 1:(AtmCovar*object@M^(i-1))){
-      		cat(sprintf("%.4f",object@Pi[j,,i]))
-    		cat("\n")
-    	}
-    	cat("\n")
-	}
+   	  cat("\n")
+   	  
+   	  if(sum(object@CMCovar)>0 | (object@Cmodel=="mtd" & object@orderVC>1) | (object@Cmodel=="mtdg" & object@orderVC>1)){
+   	    # Display the modeling of the visible transition matrix
+
+   	    cat("Modeling of the visible transition matrix : \n\n")
+   	    
+   	    cat("Vector of weights : \n")
+   	    print(object@CPhi[1,,i])
+   	    cat("\n")
+   	    
+   	    for (j in 1:(dim(object@CQ)[1])){
+   	      if (j==1){
+   	        if (dim(object@CQ)[1]==1){
+   	          cat("Transition matrix : \n")
+   	        } else {
+   	          cat("Transition matrix, lag",j,": \n")        
+   	        }
+   	      } else {
+   	        cat("Transition matrix, lag",j,": \n")        
+   	      }
+   	      print(object@CQ[j,,,i])
+   	      cat("\n")    
+   	    }
+   	    
+   	    if (sum(object@CMCovar)>0){
+   	      for (j in 1:sum(object@CMCovar)){
+   	        cat("Transition matrix for covariate",j,": \n")  
+   	        tmpcov <- as.array(object@CTCovar[[j]])
+   	        print(tmpcov[,,i])
+   	        cat("\n")  
+   	      }
+   	    }
+   	    
+   	  }
+   	  
+   	  
+   	} 
+  	
+
   	march.model.show(object)
 }
 
@@ -345,137 +436,11 @@ setMethod(f="march.nbParams",signature="march.Mtd",definition=march.mtd.nbParams
 setMethod(f="march.nbParams",signature="march.Dcmm",definition=march.dcmm.nbParams)
 
 
-###############################################################################
-# Thompson allows to compute confidence intervals according to a given model,
-# using thompson's confidence interval method describe into: Thompson, S.K. (1987) 
-# "Sample size for estimating multinomial proportions," American Statistician, 41, 42-46.
-# Adaptation to markov models is described into : Berchtold, "Confidence Intervals for Markovian Models"
-###############################################################################
+#####################################################################################
+# name methods generate a name for a march model contained in a march.model object, #
+# These methods are implemented for all models (indep, mc, mtd, dcmm).              #
+#####################################################################################
 
-#' Thompson Confidence Intervals for a march.Model.
-#' 
-#' Compute the confidence intervals using Thompson's formula on a march.Model
-#' object. See Thompson SK (1987) Sample size for estimating multinomial proportions,
-#' American Statistician 41:42-46, for details.
-#' 
-#' @param object the march.Model object on which compute the confidence intervals.
-#' @param alpha the significance level among : 0.5, 0.4, 0.3, 0.2, 0.1, 0.05, 0.025, 0.02, 0.01, 0.005, 0.001, 0.0005, 0.0001.
-#' 
-#' @return A list of half-length confidence intervals for each probability distribution of the considered model.
-#' @author Ogier Maitre, Kevin Emery
-#' @example tests/examples/march.thompson.example.R
-#' @export 
-march.thompson <- function(object,alpha){}
-
-march.model.thompson <- function(object,alpha){
-  warning("Confidence interval with thompson formula cannot be computed for abstract class \"model\", check the parameters of the call to march.thompson")
-}
-
-march.mc.thompson <- function(object,alpha){
-  d2n <- march.ci.h.d2n(alpha)
-  d <- array(NA,object@y@K^object@order)
-  
-  for( i in 1:length(d)){
-    s <- sum(object@RT[i,])
-    if( s>0 ){
-      d[i]<-sqrt(d2n/s)
-    }
-    
-  }
-  d
-}
-
-march.indep.thompson <- function(object,alpha){
-  d2n <- march.ci.h.d2n(alpha)
-  
-  sqrt(d2n/object@dsL)
-}
-
-#
-# TODO : Handle multi-sequences by computing the number of data
-#  	 influencing each distribution and summing this number.
-#
-march.mtd.thompson <- function(object, alpha){
-  d2n <- march.ci.h.d2n(alpha)
-  
-  dphi <- d2n/object@dsL
-  
-  if(dim(object@Q)[1]>1){
-    is_mtdg <- TRUE
-  }else{
-    is_mtdg <- FALSE
-  }
-  l <- march.mtd.h.n(object,object@y,is_mtdg)
-  
-  dQ <- list()
-  if(is_mtdg==FALSE){
-    dQ[[1]] <- d2n/rowSums(l$nki_0)
-  }else{
-    for(ord in 1:object@order){
-      dQ[[ord]] <- d2n/rowSums(l$nki_0[ord,,])
-    }
-  }
-  
-  dS <- list()
-  if(sum(object@MCovar)>0){
-    placeCovar <- which(object@MCovar==1)
-    for(i in 1:sum(object@MCovar)){
-      dS[[i]] <- d2n/rowSums(l$numcov[i,1:object@y@Kcov[placeCovar[i]],])
-    }
-  }
-  list(phi=dphi,Q=dQ,S=dS)
-}
-
-march.dcmm.thompson <- function(object,alpha){
-  
-  stop("This part is not yet implemented")
-  # ys <- march.dataset.h.extractSequence(object@y,1)
-  # alpha<-march.dcmm.forward(object,ys)
-  # beta<-march.dcmm.backward(object,ys)
-  # 
-  # C
-} 
-
-
-#This part create the generic method and describe how a call to this generic
-#has to be redirected to the rigth method, according to the considered object.
-setGeneric(name="march.thompson",def=function(object,alpha)march.model.thompson(object,alpha))
-
-#' This method is called with the object "march.Indep" and the aplha "numeric" and
-#' provides it to the march.thompson function.
-#' @param object contains the name of the model.
-#' @param alpha contains the Type I error
-setMethod(f="march.thompson",signature=signature(object="march.Indep",alpha="numeric"),definition=march.indep.thompson)
-
-#' This method is called with the object "march.Mc" and the aplha "numeric" and
-#' provides it to the march.thompson function.
-#'
-#' @param object contains the name of the model.
-#' @param alpha contains the Type I error
-setMethod(f="march.thompson",signature=signature("march.Mc",alpha="numeric"),definition=march.mc.thompson)
-
-#' This method is called with the object "march.Mtd" and the aplha "numeric" and
-#' provides it to the march.thompson function.
-#'
-#' @param object contains the name of the model.
-#' @param alpha contains the Type I error
-setMethod(f="march.thompson",signature=signature("march.Mtd",alpha="numeric"),definition=march.mtd.thompson)
-
-#' This method is called with the object "march.Dcmm" and the aplha "numeric" and
-#' provides it to the march.thompson function.
-#'
-#' @param object contains the name of the model.
-#' @param alpha contains the Type I error
-setMethod(f="march.thompson",signature=signature("march.Dcmm",alpha="numeric"),definition=march.dcmm.thompson)
-
-#' march.Model name.
-#' 
-#' Generate a name for the march model contained in the given \emph{object}.
-#'
-#' @param object contains the name of the model(Independence model, MTD,...). 
-#' @author Ogier Maitre & Andre Berchtold
-#' @example tests/examples/march.name.example.R
-#' @export
 march.name <- function(object){}
 
 march.model.name <- function(object){ "abstract model" } # should never be called
@@ -500,33 +465,14 @@ march.dcmm.name <- function(object){
 #This part create the generic method and describe how a call to this generic
 #has to be redirected to the right method, according to the considered object.
 setGeneric(name="march.name",def=function(object)march.model.name(object))
-
-#' This method is called with the object "march.Indep" and provides it 
-#' to the march.name function.
-#'
-#' @param object contains the name of the model.
 setMethod(f="march.name",signature=signature(object="march.Indep"),definition=march.indep.name)
-
-#' This method is called with the object "march.MC" and provides it 
-#' to the march.name function.
-#'
-#' @param object contains the name of the model.
 setMethod(f="march.name",signature=signature(object="march.Mc"),definition=march.mc.name)
-
-#' This method is called with the object "march.Mtd" and provides it 
-#' to the march.name function.
-#'
-#' @param object contains the name of the model.
 setMethod(f="march.name",signature=signature(object="march.Mtd"),definition=march.mtd.name)
-
-#' This method is called with the object "march.Dcmm" and provides it 
-#' to the march.name function.
-#'
-#' @param object contains the name of the model.
 setMethod(f="march.name",signature=signature(object="march.Dcmm"),definition=march.dcmm.name)
 
 #quote=FALSE,digits = getOption("digits")
 
+
 #' march.Model Summary.
 #'
 #' Print key values for the current model.
@@ -539,7 +485,7 @@ march.summary <- function(object,...){
   v <- array(NA,c(1,4))
   rownames(v) <- march.name(object)
   colnames(v) <- c(gettext("ll"),gettext("param"),gettext("BIC"),gettext("AIC"))
-  v[1,]<-c(object@ll,march.nbParams(object),march.BIC(object),march.AIC(object))
+  v[1,]<-c(object@ll,march.nbParams(object),march.BIC(object)@BIC,march.AIC(object)@AIC)
   
   v
 }
@@ -547,4 +493,454 @@ march.summary <- function(object,...){
 #setMethod('summary',"march.Model",march.summary)
 #setMethod(f="summary",signature=signature(object="march.Mc"),definition=march.summary)
 # setMethod(f="Summary",signature=signature(object="march.Mtd"),definition=march.summary)
-# setMethod(f="Summary",signature=signature(object="march.Dcmm"),definition=march.summary)
\ No newline at end of file
+# setMethod(f="Summary",signature=signature(object="march.Dcmm"),definition=march.summary)
+
+
+
+
+###############################################################################
+# Thompson allows to compute confidence intervals according to a given model,
+# using thompson's confidence interval method describe into: Thompson, S.K. (1987) 
+# "Sample size for estimating multinomial proportions," American Statistician, 41, 42-46.
+###############################################################################
+
+#' Thompson Confidence Intervals for an Independence model.
+#' 
+#' Compute the confidence intervals using Thompson's formula on a march.Indep
+#' object. See Thompson SK (1987) Sample size for estimating multinomial proportions,
+#' American Statistician 41:42-46, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level among : 0.5, 0.4, 0.3, 0.2, 0.1, 0.05, 0.025, 0.02, 0.01, 0.005, 0.001, 0.0005, 0.0001.
+#' 
+#' @return A list of half-length confidence intervals for each probability of the independence model.
+#' @author Ogier Maitre, Kevin Emery
+#' @example tests/examples/march.thompson.example.R
+#' @export 
+march.indep.thompson <- function(object,alpha){
+  d2n <- march.ci.h.d2n(alpha)
+  
+  d <- sqrt(d2n/object@dsL)
+  
+  # Display
+  cat("\nHalf CI for the independence model :\n")
+  cat("------------------------------------\n")
+  print(d)
+  cat("\n")
+}
+
+
+#' Thompson Confidence Intervals for a Markov chain model.
+#' 
+#' Compute the confidence intervals using Thompson's formula on a march.Mc
+#' object. See Thompson SK (1987) Sample size for estimating multinomial proportions,
+#' American Statistician 41:42-46, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level among : 0.5, 0.4, 0.3, 0.2, 0.1, 0.05, 0.025, 0.02, 0.01, 0.005, 0.001, 0.0005, 0.0001.
+#' 
+#' @return A list of half-length confidence intervals for each probability distribution of the Markov chain.
+#' @author Ogier Maitre, Kevin Emery
+#' @example tests/examples/march.thompson.example.R
+#' @export 
+march.mc.thompson <- function(object,alpha){
+  d2n <- march.ci.h.d2n(alpha)
+  d <- array(NA,object@y@K^object@order)
+  
+  for( i in 1:length(d)){
+    s <- sum(object@RT[i,])
+    if( s>0 ){
+      d[i]<-sqrt(d2n/s)
+    }
+  }
+  
+  # Display
+  cat("\nHalf CI for each row of the transition matrix :\n")
+  cat("-----------------------------------------------\n")
+  print(d)
+  cat("\n") 
+  
+}
+
+
+#' Thompson Confidence Intervals for a MTD model.
+#' 
+#' Compute the confidence intervals using Thompson's formula on a march.Mtd
+#' object. See Thompson SK (1987) Sample size for estimating multinomial proportions,
+#' American Statistician 41:42-46, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level among : 0.5, 0.4, 0.3, 0.2, 0.1, 0.05, 0.025, 0.02, 0.01, 0.005, 0.001, 0.0005, 0.0001.
+#' 
+#' @return A list of half-length confidence intervals for each probability distribution of the MTD model.
+#' @author Ogier Maitre, Kevin Emery
+#' @example tests/examples/march.thompson.example.R
+#' @export 
+march.mtd.thompson <- function(object, alpha){
+
+  d2n <- march.ci.h.d2n(alpha)
+  
+  dphi <- d2n/object@dsL
+  
+  if(dim(object@Q)[1]>1){
+    is_mtdg <- TRUE
+  }else{
+    is_mtdg <- FALSE
+  }
+  l <- march.mtd.h.n(object,object@y,is_mtdg)
+  
+  dQ <- list()
+  if(is_mtdg==FALSE){
+    dQ[[1]] <- d2n/rowSums(l$nki_0)
+  }else{
+    for(ord in 1:object@order){
+      dQ[[ord]] <- d2n/rowSums(l$nki_0[ord,,])
+    }
+  }
+  
+  # Covariates
+  dS <- list()
+  if(sum(object@MCovar)>0){
+    placeCovar <- which(object@MCovar==1)
+    for(i in 1:sum(object@MCovar)){
+      dS[[i]] <- d2n/rowSums(l$numcov[i,1:object@y@Kcov[placeCovar[i]],])
+    }
+  }
+  
+  # High-order transition matrix
+  CQ <- l$nki_0
+  
+  if(is_mtdg==FALSE){
+    # MTD model
+    
+    # Matrix of index
+    INDEX <- BuildArrayCombinations(ncol(CQ),(object@order-1),0,0)
+    
+    # Matrix of number of data
+    NHO <- matrix(0,nrow=(ncol(CQ)^object@order),ncol=ncol(CQ))
+    
+    for (i in 1:(ncol(CQ)^object@order)){
+      for (j in 1:ncol(CQ)){
+        for (k in 1:object@order){
+          NHO[i,j] <- NHO[i,j]+object@phi[k]*CQ[INDEX[i,k],j]
+        } 
+      }
+    }    
+    
+  }else{
+    # MTDg model
+    
+    # Matrix of index
+    INDEX <- BuildArrayCombinations(ncol(CQ[1,,]),(object@order-1),0,0)
+    
+    # Matrix of number of data
+    NHO <- matrix(0,nrow=(ncol(CQ[1,,])^object@order),ncol=ncol(CQ))
+    
+    for (i in 1:(ncol(CQ[1,,])^object@order)){
+      for (j in 1:ncol(CQ[1,,])){
+        for (k in 1:object@order){
+          NHO[i,j] <- NHO[i,j]+object@phi[k]*CQ[ord,INDEX[i,k],j]
+        } 
+      }
+    }  
+  }
+  
+  # CI for the high-order transition matrix
+  dNHO <- d2n/rowSums(NHO)
+  
+  
+  # Display
+  cat("\nHalf CI for the vector of weights :\n")
+  cat("-----------------------------------\n")
+  print(dphi)
+  cat("\n")
+  
+  if (dim(object@Q)[1]==1){
+    # MTD
+    cat("Half CI for each row of the transition matrix :\n")
+    cat("---------------------------------------------\n")
+    print(dQ[[1]])
+    cat("\n")
+  } else {
+    # MTDg
+    for (ord in 1:dim(object@Q)[1]){
+      cat("Half CI for each row of the transition matrix of lag",ord,":\n")
+      cat("  ------------------------------------------------------\n")
+      print(dQ[[ord]])
+      cat("\n")
+    }
+  }
+  
+  if(sum(object@MCovar)>0){
+    # Covariates
+    for (cov in 1:sum(object@MCovar)){
+      cat("Half CI for each row of the transition matrix of covariate",cov,":\n")
+      cat("  ---------------------------------------------------------\n")
+      print(dS[[cov]])
+      cat("\n")
+    }    
+  }
+  
+  
+  cat("Half CI for each row of the high-order transition matrix :\n")
+  cat("----------------------------------------------------------\n")
+  print(dNHO)
+  cat("\n")
+  
+}
+
+
+
+
+
+
+
+###############################################################################
+# Bailey allows to compute confidence intervals according to a given model,
+# using Bailey's confidence interval method describe in: Bailey, B.J.R. (1980) 
+# "Large sample simultaneous confidence intervals for the multinomial probabilities based on
+# transformation of the cell frequencies." Technometrics 1980, 22, 583–589.
+# Adaptation to Markov models is described into : Berchtold, "Confidence Intervals for Markovian Models"
+###############################################################################
+
+#' Bailey Confidence Intervals for an Independence model.
+#' 
+#' Compute the confidence intervals using Bailey's formula on a march.Indep
+#' object. See Bailey BJR (1980) Large sample simultaneous confidence intervals
+#' for the multinomial probabilities based ontransformation of the cell frequencies,
+#' Technometrics 22:583–589, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level.
+#' 
+#' @return A list of half-length confidence intervals for each probability of the independence model.
+#' @author Berchtold André
+#' @example tests/examples/march.bailey.example.R
+#' @export 
+march.indep.bailey <- function(object,alpha){
+  n <- sum(object@dsL)
+  
+  p <- array(NA,c(2,object@y@K))
+  colnames(p) <- object@y@dictionary
+  rownames(p) <- c("p-","p+")
+  for( i in 1:object@y@K){
+    ni <- object@indC[i]
+    A <- march.ci.h.A(n,ni)
+    B <- march.ci.h.B(n,ni)	
+    C <- march.ci.h.C(alpha,object@y@K,n)
+    
+    p["p-",i] = (sqrt(A)-sqrt(C*(C+1-A)))^2/(C+1)^2
+    p["p+",i] = (sqrt(B)+sqrt(C*(C+1-B)))^2/(C+1)^2
+  }
+  
+  # Display
+  cat("\nCI for the independence model :\n")
+  cat("-------------------------------\n")
+  cat("Lower bound :")
+  prmatrix(t(p["p-",]),collab=rep("",object@y@K))
+  cat("\nUpper bound :")
+  prmatrix(t(p["p+",]),collab=rep("",object@y@K))
+  cat("\n")
+  
+}
+
+
+#' Bailey Confidence Intervals for a Markov chain.
+#' 
+#' Compute the confidence intervals using Bailey's formula on a march.Mc
+#' object. See Bailey BJR (1980) Large sample simultaneous confidence intervals
+#' for the multinomial probabilities based ontransformation of the cell frequencies,
+#' Technometrics 22:583–589, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level.
+#' 
+#' @return A list of half-length confidence intervals for each probability distribution of the Markov chain.
+#' @author Berchtold André
+#' @example tests/examples/march.bailey.example.R
+#' @export 
+march.mc.bailey <- function(object,alpha){
+  
+  NHO <- object@RT
+  rNHO <- rowSums(NHO)  
+  
+  NHO.l <- matrix(NA,nrow=nrow(NHO),ncol=ncol(NHO))
+  NHO.u <- matrix(NA,nrow=nrow(NHO),ncol=ncol(NHO))
+  for (i in 1:nrow(NHO)){
+    for (j in 1:ncol(NHO)){
+      res <- march.bailey.ci(NHO[i,j],rNHO[i],alpha,ncol(NHO))
+      NHO.l[i,j] <- res[[1]]
+      NHO.u[i,j] <- res[[2]]
+    }
+  }
+  
+  # Display
+  cat("\nCI for the transition matrix :\n")
+  cat("------------------------------\n")
+  cat("Lower bound :")
+  prmatrix(NHO.l,collab=rep("",object@y@K))
+  cat("\nUpper bound :")
+  prmatrix(NHO.u,collab=rep("",object@y@K))
+  cat("\n") 
+}
+
+
+#' Bailey Confidence Intervals for a MTD model.
+#' 
+#' Compute the confidence intervals using Bailey's formula on a march.Mtd
+#' object. See Bailey BJR (1980) Large sample simultaneous confidence intervals
+#' for the multinomial probabilities based ontransformation of the cell frequencies,
+#' Technometrics 22:583–589, for details.
+#' 
+#' @param object the march.Model object on which compute the confidence intervals.
+#' @param alpha the significance level.
+#' 
+#' @return A list of half-length confidence intervals for each probability distribution of the MTD model.
+#' @author Berchtold André
+#' @example tests/examples/march.bailey.example.R
+#' @export 
+march.mtd.bailey <- function(object,alpha){
+  
+  # lag weights
+  N <- object@dsL
+  ni <- N*object@phi
+  
+  phi.l <- matrix(NA,nrow=1,ncol=object@order)
+  phi.u <- matrix(NA,nrow=1,ncol=object@order)
+  for (i in 1:object@order){
+    res <- march.bailey.ci(ni[i],N,alpha,object@order)
+    phi.l[i] <- res[[1]]
+    phi.u[i] <- res[[2]]
+  }
+  
+  # transition probabilities
+  
+  if (dim(object@Q)[1]==1){
+    
+    # A. MTD model
+    CQ <- march.mtd.h.n(object,object@y,is_mtdg=F)
+    CQ <- CQ$`nki_0`
+    rCQ <- rowSums(CQ)
+    
+    Q.l <- matrix(NA,nrow=nrow(CQ),ncol=ncol(CQ))
+    Q.u <- matrix(NA,nrow=nrow(CQ),ncol=ncol(CQ))
+    for (i in 1:nrow(CQ)){
+      for (j in 1:ncol(CQ)){
+        res <- march.bailey.ci(CQ[i,j],rCQ[i],alpha,ncol(CQ))
+        Q.l[i,j] <- res[[1]]
+        Q.u[i,j] <- res[[2]]
+      }
+    }
+    
+    # High-order matrix
+    
+    # Matrix of index
+    INDEX <- BuildArrayCombinations(ncol(CQ),(object@order-1),0,0)
+    
+    # Matrix of number of data
+    NHO <- matrix(0,nrow=(ncol(CQ)^object@order),ncol=ncol(CQ))
+    
+    for (i in 1:(ncol(CQ)^object@order)){
+      for (j in 1:ncol(CQ)){
+        for (k in 1:object@order){
+          NHO[i,j] <- NHO[i,j]+object@phi[k]*CQ[INDEX[i,k],j]
+        } 
+      }
+    }    
+  } else {
+    
+    # B. MTDg model
+    Q.l <- array(NA,dim=c(object@order,object@y@K,object@y@K))
+    Q.u <- array(NA,dim=c(object@order,object@y@K,object@y@K))
+    
+    CQ <- march.mtd.h.n(object,object@y,is_mtdg=T)
+    CQ <- CQ$`nki_0`
+    
+    for (ord in 1:object@order){
+      QCQ <- CQ[ord,,]
+      rQCQ <- rowSums(QCQ)
+      
+      for (i in 1:nrow(QCQ)){
+        for (j in 1:ncol(QCQ)){
+          res <- march.bailey.ci(QCQ[i,j],rQCQ[i],alpha,ncol(QCQ))
+          Q.l[ord,i,j] <- res[[1]]
+          Q.u[ord,i,j] <- res[[2]]
+        }
+      }
+    }
+    
+    # High-order matrix
+    
+    # Matrix of index
+    INDEX <- BuildArrayCombinations(ncol(QCQ),(object@order-1),0,0)
+    
+    # Matrix of number of data
+    NHO <- matrix(0,nrow=(ncol(QCQ)^object@order),ncol=ncol(QCQ))
+    
+    for (i in 1:(ncol(QCQ)^object@order)){
+      for (j in 1:ncol(QCQ)){
+        for (k in 1:object@order){
+          NHO[i,j] <- NHO[i,j]+object@phi[k]*CQ[k,INDEX[i,k],j]
+        } 
+      }
+    }    
+  }
+  
+  # CI for the high-order transition matrix
+  rNHO <- rowSums(NHO)  
+  
+  NHO.l <- matrix(NA,nrow=nrow(NHO),ncol=ncol(NHO))
+  NHO.u <- matrix(NA,nrow=nrow(NHO),ncol=ncol(NHO))
+  for (i in 1:nrow(NHO)){
+    for (j in 1:ncol(NHO)){
+      res <- march.bailey.ci(NHO[i,j],rNHO[i],alpha,ncol(NHO))
+      NHO.l[i,j] <- res[[1]]
+      NHO.u[i,j] <- res[[2]]
+    }
+  }
+  
+  # Display
+  cat("\nCI for the vector of weights :\n")
+  cat("------------------------------\n")
+  cat("Lower bound :\n")
+  print(phi.l[1,])
+  cat("\nUpper bound :\n")
+  print(phi.u[1,])
+  cat("\n")
+  
+  if (dim(object@Q)[1]==1){
+    # MTD
+    cat("CI for the transition matrix :\n")
+    cat("------------------------------\n")
+    cat("Lower bound :")
+    prmatrix(Q.l,collab=rep("",object@y@K))
+    cat("\nUpper bound :")
+    prmatrix(Q.u,collab=rep("",object@y@K))
+    cat("\n")
+  } else {
+    # MTDg
+    for (ord in 1:dim(object@Q)[1]){
+      cat("CI for the transition matrix of lag",ord,":\n")
+      cat("---------------------------------------\n")
+      cat("Lower bound :")
+      prmatrix(Q.l[ord,,],collab=rep("",object@y@K))
+      cat("\nUpper bound :")
+      prmatrix(Q.u[ord,,],collab=rep("",object@y@K))
+      cat("\n")
+    }
+  }
+  
+  cat("CI for the high-order transition matrix :\n")
+  cat("-----------------------------------------\n")
+  cat("Lower bound :")
+  prmatrix(NHO.l,collab=rep("",object@y@K))
+  cat("\nUpper bound :")
+  prmatrix(NHO.u,collab=rep("",object@y@K))
+  cat("\n")
+  
+  #list(phi.l,phi.u,Q.l,Q.u,NHO.l,NHO.u)
+}
+
+
+
+
